import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_authentication/senior_authentication.dart';
import 'package:senior_authentication/src/screens/password/bloc/password_form_state.dart';
import 'package:senior_authentication/src/utils/exceptions.dart';

part 'password_event.dart';
part 'password_state.dart';

class PasswordBloc extends Bloc<PasswordEvent, PasswordState> {
  final AuthenticationRepository authenticationRepository;
  final String username;

  PasswordBloc({required this.authenticationRepository, required this.username})
      : super(PasswordState(username: username)) {
    on<PasswordVisibilityChange>((event, emit) {
      emit(state.copyWith(passwordVisible: !state.passwordVisible));
    });

    on<PasswordChanged>((event, emit) {
      emit(state.copyWith(password: event.password, formState: const PasswordFormInitial()));
    });

    on<PasswordSubmitted>((event, emit) async {
      emit(state.copyWith(formState: PasswordFormLoadInProgress()));

      try {
        var userLogin = UserLogin(username: state.username, password: state.password);

        await authenticationRepository.logInWithEmailAndPassword(userLogin);

        emit(state.copyWith(formState: PasswordFormLoadSuccess()));
      } on LogInWithEmailAndPasswordIncorrect catch (_) {
        emit(state.copyWith(formState: PasswordFormIncorrectPassword()));
      } catch (_) {
        emit(state.copyWith(formState: PasswordFormLoadFailure(Exception('Erro na autenticação'))));
      }
    });
  }
}
