import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_authentication/senior_authentication.dart';
import 'package:senior_authentication/src/utils/exceptions.dart';
import './username_form_state.dart';

part 'username_event.dart';
part 'username_state.dart';

class UsernameBloc extends Bloc<UsernameEvent, UsernameState> {
  final AuthenticationRepository authenticationRepository;

  UsernameBloc({required this.authenticationRepository}) : super(const UsernameState()) {
    on<UsernameChanged>((event, emit) {
      emit(state.copyWith(username: event.username, formState: const UsernameFormInitial()));
    });

    on<UsernameSubmitted>((event, emit) async {
      emit(state.copyWith(formState: UsernameFormLoadInProgress()));

      try {
        var domain = state.username.split('@')[1];
        var tenantLoginSettings = await authenticationRepository.getTenantLogInSettings(domain, null);

        emit(
          state.copyWith(
            formState: UsernameFormLoadSuccess(username: state.username, tenantLoginSettings: tenantLoginSettings),
          ),
        );
      } on TenantLogInSettingsIncorrect catch (err) {
        emit(state.copyWith(formState: UsernameFormLoadFailure(err)));
      } catch (_) {
        emit(state.copyWith(formState: UsernameFormLoadFailure(Exception('Erro na autenticação'))));
      }
    });
  }
}
